@@ -173,6 +173,7 @@ class SniTunServerWorker(Thread):
             self._workers.append(worker)
 
         self._server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self._server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         self._server.bind((self._host, self._port))
         self._server.setblocking(False)
         self._server.listen(80 * 1000)
@@ -228,7 +229,7 @@ class SniTunServerWorker(Thread):
                 else:
                     self._poller.unregister(fileno)
                     con = connections.pop(fileno)
-                    con.close()
+                    self._close_socket(con, shutdown=False)
 
             # cleanup stale connection
             for fileno in tuple(stale):
@@ -237,7 +238,7 @@ class SniTunServerWorker(Thread):
                 elif stale[fileno] >= WORKER_STALE_MAX:
                     self._poller.unregister(fileno)
                     con = connections.pop(fileno)
-                    con.close()
+                    self._close_socket(con)
                 else:
                     stale[fileno] += 1
 
@@ -255,13 +256,12 @@ class SniTunServerWorker(Thread):
             data = con.recv(2048)
         except OSError as err:
             _LOGGER.warning("Receive fails: %s", err)
-            con.close()
+            self._close_socket(con, shutdown=False)
             return
 
         # No data received
         if not data:
-            with suppress(OSError):
-                con.shutdown(socket.SHUT_RDWR)
+            self._close_socket(con)
             return
 
         # Peer connection
@@ -273,8 +273,7 @@ class SniTunServerWorker(Thread):
         # TLS/SSL connection
         if data[0] != 0x16:
             _LOGGER.warning("No valid ClientHello found: %s", data)
-            with suppress(OSError):
-                con.shutdown(socket.SHUT_RDWR)
+            self._close_socket(con)
             return
 
         try:
@@ -291,5 +290,12 @@ class SniTunServerWorker(Thread):
                 return
             _LOGGER.debug("No responsible worker for %s", hostname)
 
+        self._close_socket(con)
+
+    @staticmethod
+    def _close_socket(con: socket.socket, shutdown: bool = True) -> None:
+        """Gracefull shutdown a socket or free the handle."""
         with suppress(OSError):
-            con.shutdown(socket.SHUT_RDWR)
+            if shutdown:
+                con.shutdown(socket.SHUT_RDWR)
+            con.close()
